
#include <libglademm/xml.h>
#include <libglademm/private/xml_p.h>

/* $Id: xml.ccg,v 1.7 2002/04/29 08:59:45 murrayc Exp $ */


/* xml.ccg
 * 
 * Copyright (C) 2002 The libglademm Development Team
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <glade/glade-xml.h>

namespace Gnome
{

namespace Glade
{

Xml::Xml(const Glib::ustring& filename, const Glib::ustring& root, const Glib::ustring& domain)
: Glib::Object((GObject*)(g_object_new(get_type(), (char*)0)))
{ 
  //Copied from glade_xml_new():                                                
  glade_xml_construct(gobj(), filename.c_str(), root.c_str(), domain.c_str());     
}

Xml::Xml(const Glib::ustring& filename, const Glib::ustring& root)
: Glib::Object((GObject*)(g_object_new(get_type(), (char*)0)))
{ 
  glade_xml_construct(gobj(), filename.c_str(), root.c_str(), 0);     
}

Xml::Xml(const Glib::ustring& filename)
: Glib::Object((GObject*)(g_object_new(get_type(), (char*)0)))
{ 
  glade_xml_construct(gobj(), filename.c_str(), 0, 0);     
}


Glib::RefPtr<Xml> Xml::create_from_buffer(const Glib::ustring& buffer, const Glib::ustring& root, const Glib::ustring& domain) //static
{
  return Glib::wrap( glade_xml_new_from_buffer(buffer.c_str(), buffer.size(), root.c_str(), domain.c_str()) );
}
Glib::RefPtr<Xml> Xml::create_from_buffer(const Glib::ustring& buffer, const Glib::ustring& root) //static
{
  return Glib::wrap( glade_xml_new_from_buffer(buffer.c_str(), buffer.size(), root.c_str(), 0) );
}
Glib::RefPtr<Xml> Xml::create_from_buffer(const Glib::ustring& buffer) //static
{
  return Glib::wrap( glade_xml_new_from_buffer(buffer.c_str(), buffer.size(), 0, 0) );
}

void Xml::reparent_widget(const Glib::ustring& name, Gtk::Container& container)
{
  Gtk::Widget* pWidget = 0;
  get_widget(name, pWidget);
  if(pWidget)
    pWidget->reparent(container);
}

//Custom coded so that we can custom-code the vfunc in the Xml class.
//This is marginally better than modifying gtkmmproc to allow this.
GType Xml_Class::lookup_type_vfunc_callback(GladeXML* self, const gchar* classname)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      return obj->lookup_type(Glib::convert_const_gchar_ptr_to_ustring(classname));
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
    );

    g_assert(base != 0);

    if(base->lookup_type)
      (*base->lookup_type)(self, classname);
    }

  typedef GType RType;
  return RType();
}

GType Xml::lookup_type(const Glib::ustring& classname)
{
  //See if there is a gtkmm version of the gclass:
  Glib::ustring classname_prefixed = "gtkmm__" + classname;   //gtkmm uses a prefix.
  GType gtype = g_type_from_name(classname_prefixed.c_str());

  if (gtype == G_TYPE_INVALID) //If it's not a registered typename
  {
    g_warning("murray debug: fallback: %s\n", classname.c_str());
    //There's no gtkmm derived type, so just use the normal one.
    gtype = g_type_from_name(classname.c_str());
  }

  return gtype;
}


} /* namespace Glade */
} /* namespace Gnome */


namespace
{
} // anonymous namespace


namespace Glib
{

Glib::RefPtr<Gnome::Glade::Xml> wrap(GladeXML* object, bool take_copy /* = false */)
{
  return Glib::RefPtr<Gnome::Glade::Xml>( dynamic_cast<Gnome::Glade::Xml*> (Glib::wrap_auto ((GObject*)(object), take_copy)) );
  //We use dynamic_cast<> in case of multiple inheritance.
}

} /* namespace Glib */


namespace Gnome
{

namespace Glade
{


/* The *_Class implementation: */

GType Xml_Class::get_type()
{
  if(!gtype_) // create the GType if necessary
  {
    // TODO: This is currently just optimized away, apparently with no harm.
    // Is it actually needed?
    // Make sure that the parent type has been created.
    CppClassParent::CppObjectType::get_type();

    // Create the wrapper type, with the same class/instance size as the base type.
    register_derived_type(glade_xml_get_type(), (GClassInitFunc) &class_init_function);

    // Add derived versions of interfaces, if the C type implements any interfaces:
  }

  return gtype_;
}

void Xml_Class::class_init_function(BaseClassType* klass)
{
  CppClassParent::class_init_function((CppClassParent::BaseClassType*) klass);
  klass->lookup_type = &lookup_type_vfunc_callback;;
  }


Glib::ObjectBase* Xml_Class::wrap_new(GObject* o)
{
  return new Xml((GladeXML*)(o));
}


/* The implementation: */

GladeXML* Xml::gobj_copy()
{
  reference();
  return gobj();
}

Glib::RefPtr<Xml> Xml::wrap_specific_type(GladeXML* gobject, bool take_copy /* = false */) //static
{
  Glib::RefPtr<Xml> refPtr;

  if(gobject)
  {
    //Check for an existing wrapper:
    Xml* pCppObject = dynamic_cast<Xml*>(Glib::ObjectBase::_get_current_wrapper(G_OBJECT(gobject)));
    if(pCppObject)
    {
      //Return the existing wrapper:
      refPtr = Glib::RefPtr<Xml>(pCppObject);
    }
    else
    {
      //Create a new wrapper:
      refPtr = Glib::RefPtr<Xml>( new Xml(gobject) );
    }

    if(take_copy && refPtr)
      refPtr->reference();
  }

  return refPtr;
}

Xml::Xml(GladeXML* castitem)
: Glib::Object((GObject*)(castitem))
{}

Xml::~Xml()
{
}

Xml::CppClassType Xml::xml_class_; //Initialize static member.

GType Xml::get_type()
{
  return xml_class_.get_type();
}

GType Xml::get_base_type()
{
  return glade_xml_get_type();
}

Glib::RefPtr<Xml> Xml::create(const Glib::ustring& filename)
{
  return Glib::RefPtr<Xml>( new Xml(filename) );
}
Glib::RefPtr<Xml> Xml::create(const Glib::ustring& filename, const Glib::ustring& root)
{
  return Glib::RefPtr<Xml>( new Xml(filename, root) );
}
Glib::RefPtr<Xml> Xml::create(const Glib::ustring& filename, const Glib::ustring& root, const Glib::ustring& domain)
{
  return Glib::RefPtr<Xml>( new Xml(filename, root, domain) );
}
Gtk::Widget* Xml::get_widget(const Glib::ustring& name)
{
  return Glib::wrap(glade_xml_get_widget(gobj(), name.c_str()));
}

Glib::ListHandle<Gtk::Widget*> Xml::get_widget_prefix(const Glib::ustring& name)
{
  return Glib::ListHandle<Gtk::Widget*>(glade_xml_get_widget_prefix(gobj(), name.c_str()), Glib::OWNERSHIP_SHALLOW);
}

Glib::ustring Xml::relative_file(const Glib::ustring& filename)
{
  return Glib::convert_return_gchar_ptr_to_ustring(glade_xml_relative_file(gobj(), filename.c_str()));
}


} // namespace Glade

} // namespace Gnome

